#!/usr/local/bin/tclsh
# Script to permit user identification for dpgate enhanced features
# Require tcl 8.3 (with tclsh interpreter into /usr/local/bin)
#
# Copyright (c) 2000 Paolo Subiaco iw3grx@ir3ip.ampr.org
# http://gw.ir3ip.ampr.org/dpbox
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Iw3grx - dp.tcl v.0.4 - November 13, 2000 
#
# require:
# /usr/local/bin/tclsh (tcl interpreter 8.3 or above)
# gzip compress program, executable from httpd user (nobody?)
#
# user definitions:
set DOMAIN "ir3ip.ampr.org"                            ;# host domain (used in cookie: don't set to ampr.org !)
set HTDOCS_DIR "/home/www/html"                        ;# Webserver HTML directory
set DPBOX_HTTP_DIR "/dpbox"
set DPBOX_COUNTER ${HTDOCS_DIR}${DPBOX_HTTP_DIR}/counter/dp.counter        ;# DPBOX counter filename (the directory must be RW to httpd user, nobody)
set DP_LOGFILE "/home/httpd/logs/dpaccess.log"         ;# DPBOX LOG FILE
set DPACCESS_PATH "/cgi-bin/dp.tcl"                    ;# URL PATH where is stored this cgi script (ex /cgi-bin/dp.tcl
set DPACCESS_URL "http://gw.ir3ip.ampr.org"            ;# Primary website URL
set DPBOX_ADMIN "webmaster@ir3ip.ampr.org"             ;# DPBOX CGI administrator (email address)
set DPBOX_ACCESS_PROG "/usr/local/bin/dpaccess_pass"   ;# Program that en/decrypt passwords
set COOKIE_EXPIRE_DAYS 30                              ;# Number of days after which cookie expires
set DP_SOCKET "/var/run/dpbox"                         ;# DPBOX socket file
set DP_CODECONV "/usr/local/bin/dpcodeconv"            ;# dpcodeconv file
set DP_PRIVLINKS "/usr/local/bin/dpprivlinks"          ;# dpprivlinks file (developed by iw3grx)
set DP_GATE "/usr/local/bin/dpgate"                    ;# dpgate file (modified by iw3grx)
set DPBOX_CALLSIGN "IK3GET"
set MTA_PATH  "/usr/lib/sendmail"                      ;# MTA path
set DP_HOLDADMIN "iw3grx@ir3ip.ampr.org"               ;# Who receive mail for each sent mail: sintax is "user1@host1 user2@host2 ...."
set default_language "en"        ;# default language = en (english)
set default_style "default"


# start of script!
set gzip 0          ;# 1 if browser support gzip encoding
set fdout stdout
set name ""
set par(lasttime) 0
set fdlang 0
set nlang 0
set style ""

######### puts language text ################
proc putslang {n} {
    global fdlang nlang
    if "$n <= $nlang" {
	seek $fdlang 0 start
    }
    while {[gets $fdlang buffer]>0} {
	if {[scan $buffer "%d" nlang]==1 && "$nlang"=="$n"} {
	    set len [string length $buffer]
	    for {set i 0} {$i<$len && [string is digit [string index "$buffer" $i]]} {incr i} {}
	    return [string trim [string range "$buffer" $i end]]
	}
    }
    return ""
}

######## send email to dpaccess admins #######
proc sendmail {subject message} {
    global par DP_HOLDADMIN DPACCESS_PATH MTA_PATH
    if {[catch {open "|$MTA_PATH $DP_HOLDADMIN" WRONLY} fd]} {
	puts stderr "$DPACCESS_PATH: error opening Mail Transport Agent $MTA_PATH\nError: $fd"
	return
    }
    puts $fd "From: dpaccess_script\nTo: $DP_HOLDADMIN\nSubject: $subject \n\n$message"
    close $fd
}

######### print statistics ###################
proc printstat {} {
    global gzip env par DP_LOGFILE
    
    if "! [catch "open $DP_LOGFILE {WRONLY APPEND CREAT}" fd]" {
	puts -nonewline $fd "[clock format [clock seconds] -format  "%d/%m/%Y %T"] "
	if {[info exist env(HTTP_USER_AGENT)]} {
	    
	    puts -nonewline $fd [format "%-50s : " [string range $env(HTTP_USER_AGENT) 0 49]]
	}
	if {[info exist env(REMOTE_ADDR)]} {
	    set ip [split $env(REMOTE_ADDR) "."]
	    puts -nonewline $fd [format "%03d.%03d.%03d.%03d " [lindex $ip 0] [lindex $ip 1] [lindex $ip 2] [lindex $ip 3]]
	}
	if "$gzip" { puts -nonewline $fd "GZIP " } else { puts -nonewline $fd "     " }
	if {[info exist par(u)]} {
	    puts -nonewline $fd "[format %6s $par(u)] "
	} else {
	    puts -nonewline $fd "NoCall "
	}
	if {[info exist par(c)]} {
	    set params ""
	    if {[string equal listb "$par(c)"]} {
		set params "board=$par(b)"
	    } elseif {[string equal BR "$par(c)"] || [string equal rep "$par(c)"] || [string equal com "$par(c)"]} {
		set params "board=$par(b) num=$par(n)"
	    } elseif {[string equal send "$par(c)"]} {
		set params "$par(board)@$par(zone) subject=$par(subject)"
		sendmail "new message from $par(u)" "New message was sent at [clock format [clock seconds] -format "%d/%m/%Y %T"]\nFrom:$par(u) To:$params\n"
	    } elseif {[string equal sendrep "$par(c)"]} {
		set params "board=$par(b) num=$par(n)"
#		sendmail $params
	    } elseif {[string equal sendcom "$par(c)"]} {
		set params "board=$par(b) num=$par(n)"
		sendmail $params
	    } elseif {[string equal bbsinfo "$par(c)"]} {
		set params "bbs=$par(bbs)"
	    } elseif {[string equal userinfo "$par(c)"]} {
		set params "user=$par(user)"
	    } elseif {[string equal seekh "$par(c)"]} {
		set params "pattern=$par(key)"
	    }
	    puts -nonewline $fd "command=$par(c) $params "
	}
	puts $fd "" ;#just to send carriage return
	close $fd
    } 
    exit
}

############ print cgi header, with cookie (if setted) #######################
proc printheader {cookie} {
    global gzip fdout env DOMAIN DPACCESS_PATH COOKIE_EXPIRE_DAYS DPBOX_HTTP_DIR style
    # Set cookie ?
    if {[string length $cookie]>0} {
	puts "Set-Cookie: $cookie; expires=[clock format [expr [clock seconds]+86400*$COOKIE_EXPIRE_DAYS] -format "%A, %d-%b-%Y %T GMT" -gmt 1]"
	puts "Date: [clock format [clock seconds] -format "%a, %d %b %Y %T GMT" -gmt 1]"
    }
    puts "Content-Type: text/html"
    # Can I compress page using gzip ?
    if {[info exist env(HTTP_ACCEPT_ENCODING)] && [string first "gzip" $env(HTTP_ACCEPT_ENCODING)]>=0} {
	set gzip 1
    }
    if "$gzip==1" {
	if {[catch {open {|gzip -c -f } {WRONLY}} fdout]} {
	    set fdout stdout 
	    puts "\n"
	} else {
	    puts "Content-Encoding: gzip"
	    puts ""
	}
    } else {
	puts "\n"
    }
    # Now send to fdout the HTML page header (Modify it as you prefer!!)
    puts $fdout "<html><head><title>[putslang 1]</title>\
<link rel=\"stylesheet\" href=\"$DPBOX_HTTP_DIR/$style.css\"></head>"
    puts $fdout "<body class=\"main\">\
<table cols=3 width=\"100%\"><tr align=center>\
<td><img src=\"${DPBOX_HTTP_DIR}/mailwld.gif\">\
<td align=\"center\"><p class=\"head1\">[putslang 901]</p>\
<p class=\"head2\">[putslang 902]</p>\
<td><img src=\"${DPBOX_HTTP_DIR}/extra.gif\" align=right></table>\
<br><center>"
}

######## print footer ##############
proc printfooter {level} { ;# level is the number of level user want to return back
    global counter fdout
    puts $fdout "<center><TABLE class=select><TR>\
<TD><A HREF=\"JavaScript:window.history.go($level)\">[putslang 2]</A>\
<TD><A HREF=/>[putslang 3]</A><tr>\
<td class=\"counter\" colspan=2>[putslang 4] $counter [putslang 5]</table></center></BODY></HTML>"
    close $fdout
    printstat
}

########### Show the form to permit user authentication ##########
proc askpassword {} {
    global fdout DPBOX_ADMIN DPACCESS_PATH
    # Please modify the text below!
    puts $fdout "<form action=\"$DPACCESS_PATH\" method=post><table class=\"auth\" width=\"60%\" border=\"3\">\
<TR><TD COLSPAN=2 ALIGN=CENTER>[putslang 6]\
<TR><TD>[putslang 7] <TD><INPUT TYPE=TEXT NAME=u SIZE=20 MAXLENGTH=6>\
<TR><TD>[putslang 8] <TD><INPUT TYPE=PASSWORD NAME=p SIZE=20 MAXLENGTH=20>\
<TR><TD COLSPAN=2 ALIGN=CENTER><INPUT TYPE=submit></table>\
<input type=\"hidden\" name=\"c\" value=\"control\"></form>\
<table class=\"auth\" width=\"60%\" border=\"3\"> 
<tr><td>[putslang 9]<br><br>
[putslang 10]<br><br>[putslang 11]\
<tr><td align=\"center\"><form action=\"$DPACCESS_PATH\" method=\"post\">\
<input type=\"hidden\" name=\"c\" value=\"regform\">\
<input type=\"submit\" value=\"[putslang 12]\"></form></table>\
</body></html>"
    close $fdout
    printstat
}

############ Show the list of available stylesheets ##############
proc getstyles {} {
    global HTDOCS_DIR DPBOX_HTTP_DIR fdout style message
    # lists all *.css files withing dpbox dir.
    set styles [lsort [glob -path "${HTDOCS_DIR}${DPBOX_HTTP_DIR}/" -types {f r} "*.css"]]
    set message "<select name=\"newstyle\">"
    for {set i 0} {$i<[llength $styles]} {incr i} {
	set stylename [string range [file tail [lindex $styles $i]] 0 [expr [string first ".css" [file tail [lindex $styles $i]]]-1]] 
	append message {<option}
	if {[string equal "$stylename" "$style"]} {append message { selected}}
	append message ">$stylename"
    }
    append message {</select>}
    return "$message"
}


############ Show the table within menu ##############
proc printmenu {} {
    global fdout DPBOX_HTTP_DIR DPACCESS_PATH DPBOX_CALLSIGN par name
    if {[expr [clock seconds]-$par(lasttime)]>7200} {
	puts $fdout "<big>[putslang 52] $name [putslang 53] $DPBOX_CALLSIGN [putslang 54]</big>"
    }
    puts $fdout "<TABLE class=\"menu\" BORDER=1 CELLSPACING=5>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=listp>[putslang 14]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=check>[putslang 15]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=check100>[putslang 16]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=check500>[putslang 17]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=dir>[putslang 18]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=create>[putslang 19]</A></TD></TR>"
    puts $fdout "<TR><TD><FORM ACTION=$DPACCESS_PATH METHOD=post>[putslang 20]</TD>\
<TD><INPUT TYPE=hidden NAME=c VALUE=seekh><INPUT SIZE=10 MAXLENGTH=20 NAME=key><INPUT TYPE=submit VALUE=\"[putslang 21]\"></FORM></TD></TR>"
    puts $fdout "<TR><TD><FORM ACTION=${DPACCESS_PATH} METHOD=post>[putslang 24] [putslang 22]<BR>([putslang 23]) </TD>\
<TD><INPUT TYPE=hidden NAME=c VALUE=bbsinfo><INPUT SIZE=10 MAXLENGTH=10 name=bbs><INPUT TYPE=submit VALUE=[putslang 24]></FORM></TD></TR>\
<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=u>[putslang 25]</A></TD></TR>"
    puts $fdout "<TR><TD><FORM ACTION=$DPACCESS_PATH METHOD=post>[putslang 24] [putslang 26]</TD>\
<TD><INPUT TYPE=hidden NAME=c VALUE=userinfo><INPUT SIZE=10 MAXLENGTH=10 NAME=user><INPUT TYPE=submit VALUE=\"[putslang 24]\"></FORM></TD></TR>"
    puts $fdout "<TR><TD COLSPAN=2><A HREF=$DPACCESS_PATH?c=v>[putslang 27]</A></TD></TR>\
<TR><TD COLSPAN=2><A HREF=${DPBOX_HTTP_DIR}/dp_ita.html>[putslang 28]</A>\
<TR><TD COLSPAN=2><A HREF=/pub/newbin>[putslang 29]</A>\
<TR><TD COLSPAN=2><A HREF=/pub>[putslang 30]</A>"
    puts $fdout "<tr><td>[putslang 55] [putslang 56]<td>\
<form action=\"$DPACCESS_PATH\" method=post>[getstyles]<input type=submit value=\"[putslang 55]\"><input type=\"hidden\" name=\"c\" value=\"selstyle\"></form>\
</TABLE></CENTER>"
    puts $fdout "<H2>[putslang 31]</H2><ul>\
<li><a href=\"http://ir2ip.archivio.it/cgi-bin/dp.tcl\">IW2LSI \[Bergamo - Italy\]</a><br>\
<li><a href=\"http://db0sif.ernaehrung.uni-giessen.de/boxmain.html\">DB0SIF \[Giessen - Germany\]</a><br>\
<li><a href=\"http://gw.hb9ww.ampr.org/dpbox/boxmain.html\">HB9WW \[Le Locle - CH\]</a><br>\
<li><a href=\"http://db0exp.de/dpindex.html\">DB0EXP \[Leipzig - Germany\]</a><br>\
<li><a href=\"http://lx0net.guddland.lu/dpbox/boxmain.html\">LX0NET \[Rodange - Luxembourg\]</a><br>\
</ul><br><br>"
    puts $fdout "<small>dpaccess script v1.0 developed by iw3grx <a href=\"http://gw.ir3ip.ampr.org/iw3grx\">http://gw.ir3ip.ampr.org/iw3grx</a></small></body></html>"
    close $fdout
    printstat
}
    

########### An error occurred before sending HTML page: write this error in HTML format ##############
proc printerror {string} {
    # write error output to stdout in a html page
    global DPBOX_HTTP_DIR style
    puts "Content-Type: text/html\n\n<HTML><HEAD><TITLE>DPBOX private access</TITLE><link rel=\"stylesheets\" href=\"${DPBOX_HTTP_DIR}/${style}.css\"></HEAD><BODY class=\"error\"><CENTER>\
This ERROR was generated by DPBOX private access script:<BR>\
<p class=\"errno\">$string</p></CENTER></BODY></HTML>"
    exit
}


####### Parse CGI parameters stored in message ##############
proc parsepar {delim message} {
    global par
    foreach query [split $message "$delim"] {
	set index [string first "=" $query]
	set parname [string range $query 0 [expr $index-1]]
	set parvalue [string map {"%25" "+++"} [string map {"+" " "} [string range $query [expr $index+1] end]]]
	while 1 {
	    set index [string first "%" $parvalue]
	    if "$index == -1" {
		break
	    } else {
		set hex [string range $parvalue [expr $index+1] [expr $index+2]]
		regsub -all -nocase "\%$hex" $parvalue [format "%c" "0x$hex"] parvalue
	    }
	}
	set parvalue [string map {"+++" "%"} $parvalue]
	set par($parname) $parvalue
    }
}

########### Access control (see dpaccess.txt) #############
proc access {command user arg} {
    global DPBOX_ACCESS_PROG DPBOX_ADMIN fdout name
    if "[catch "exec $DPBOX_ACCESS_PROG $command $user $arg" result]" {
	printheader ""
	puts $fdout "<BIG>[putslang 32]<BR>\
<A HREF=mailto:$DPBOX_ADMIN>[putslang 33]</A></BIG></BODY></HTML>"
	puts stderr "DP.TCL: error running program $DPBOX_ACCESS_PROG: argv1=$command argv2=$user argv3=$arg result=$result"
	close $fdout
	printstat
    }
    if "[string equal "control" "$command"]" {
	if "[string equal -length 3 "Ok " "$result"]" {
	    set name [string range $result 2 end]
	    return $result
	} elseif "[string equal "No-User" "$result" ]" {
	    printheader ""
	    puts $fdout "<BIG>[putslang 34]</BIG><BR>[putslang 35]<BR>"
	    puts stderr "DP.TCL: $DPBOX_ACCESS_PROG $command $user $arg -> $result"
	    askpassword
	} elseif "[string equal "No-Pass" "$result"]" {
	    printheader ""
	    puts $fdout "<BIG>[putslang 36]</BIG><BR>[putslang 37]<BR>"
	    puts stderr "DP.TCL: $DPBOX_ACCESS_PROG $command $user $arg -> $result"
	    askpassword
	}
    } elseif "[string equal "encode" "$command"]" {
	return $result
    } elseif "[string equal "decode" "$command"]" {
	if "[string equal -length 3 "Ok " "$result"]" {
	    set name [string range $result 2 end]
	    return 
	} else {
	    puts $fdout "<BIG>[putslang 38]</BIG><BR>[putslang 39]<BR>"
	    puts stderr "DP.TCL: $DPBOX_ACCESS_PROG $command $user $arg -> $result"
	    askpassword
	}
    }
}

proc verifyaccess {} {
    global par
    set result [access control $par(u) $par(p)]
    if {[string equal -length 2 "Ok" "$result"]} {
	# set the cookie
	printheader "DP=$par(u):[access encode "$par(u)" "blablabla"]:[clock seconds]"
	printmenu
    } else {
	printheader ""
	askpassword
    }
}

### Parse message body, splitting long lines into "max-79chars lines" ###
proc parsebody {body} {
    global message
    set i 0
    set len [string bytelength $body]
    set message ""
    while {1} {
	set j [string first "\n" "$body" $i]
	if {$j==-1} {#fine del body
	    set j $len
	}
	if {[expr $j-$i]>79} {
	    #split line
	    set k $i
	    while {1} {
		set j [string first { } "$body" [expr $k+1]]
		if {[expr $j-$i]<=80 && $j!=-1} {
		    set k $j
		} else {
		    break
		}			  
	    }
	    # k = posizione fino a cui stampare.
	    if {$k==$i} {
		#linea troppo lunga (senza spazi)
		set j [expr $i+78]
		set message [format "%s%s\n" "$message" [string range "$body" $i $j]]
		set i [expr $j+1]
	    } else {
		set j [expr $k-1]
		set message [format "%s%s\n" "$message" [string range "$body" $i $j]]
		set i [expr $j+2]
		while {[string equal [string index "$body" $i] " "]} {
		    incr i
		}
	    }
	} else {
	    set message [format "%s%s\n" "$message" [string range "$body" $i [expr $j-1]]]
	    if {[expr $j+1]>=$len} {
		break
	    } else {
		set i [expr $j+1]
	    }
		
	}
    }
}


################ provide registration form #####################
proc regform {} {
    global par fdout DPBOX_ADMIN DPACCESS_PATH
    # Please modify the text below!
    puts $fdout "<form action=\"$DPACCESS_PATH\" method=\"post\">\
<table class=\"auth\" width=\"60%\">\
<th colspan=\"2\">[putslang 57]</th>
<tr><td colspan=\"2\">[putslang 58] <a href=\"mailto:${DPBOX_ADMIN}\">${DPBOX_ADMIN}</a>\
<tr><td>[putslang 7] <td><input type=\"text\" name=\"u\""
    if {[info exist par(u)]} { puts $fdout " value=\"$par(u)\"" }
    puts $fdout " size=\"20\" maxlength=\"6\">\
<tr><td>[putslang 8] <td><input type=\"password\" name=\"p\""
    if {[info exist par(p)]} { puts $fdout " value=\"$par(p)\"" }
    puts $fdout "size=\"20\" maxlength=\"20\">\
<tr><td>[putslang 59] <td><input type=\"text\" name=\"firstname\" size=\"20\" maxlength=\"30\">\
<tr><td>[putslang 60] <td><input type=\"text\" name=\"lastname\" size=\"20\" maxlength=\"30\">\
<tr><td>[putslang 61] <td><input type=\"text\" name=\"qth\" size=\"20\" maxlength=\"60\">\
<tr><td>[putslang 62] <td><input type=\"text\" name=\"email\" size=\"20\" maxlength=\"60\">\
<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" value=\"[putslang 12]\">\
<tr><td colspan=\"2\">[putslang 63]</table>\
<input type=\"hidden\" name=\"c\" value=\"register\"></form></body></html>"
    close $fdout
    printstat
}

############ User informations sent to DPBOX_ADMIN ##################
proc registerok {} {
    global par fdout 
    puts $fdout "<div class=\"auth\"><big>[putslang 64]</big><br><br><a href=\"/\">[putslang 3]</a><br></div></body></html>"
}

#################################### Main ##############################
# read input cgi parameters
if {![info exist env(REQUEST_METHOD)]} {
    printerror "Invalid cgi parameters (REQUEST_METHOD not set)"
    return
}
set message $env(QUERY_STRING)
if {[string length $message]>0} {
    parsepar "&" $message
}
if {[string equal $env(REQUEST_METHOD) "POST"]} {
    set message [read stdin $env(CONTENT_LENGTH)]
    parsepar "&" $message
}
# read the cookie (if present)
if {[info exist env(HTTP_COOKIE)]} {
    set message [string map {"; " ";"} $env(HTTP_COOKIE)]
    parsepar ";" $message
}
# now open the appropriate language file
set lang ""
if {[info exist env(HTTP_ACCEPT_LANGUAGE)]} {
    foreach message [split $env(HTTP_ACCEPT_LANGUAGE) ","] {
	if {![catch {open [format "%s%s/dptext.%s" "$HTDOCS_DIR" "$DPBOX_HTTP_DIR" [string range $message 0 1]]  RDONLY} fdlang]} {
	    set lang [string range $message 0 1]
	    break
	}
    }
}
if {[string length $lang]==0} {
    set lang "$default_language"
    set fdlang [open "$HTDOCS_DIR$DPBOX_HTTP_DIR/dptext.$lang" RDONLY]
}

# which stylesheet selected?
if {[info exist par(STYLE)]} {set style "$par(STYLE)"} else { set style "$default_style"}

# does counter filename exist ?
if {[file readable $DPBOX_COUNTER] && [file writable $DPBOX_COUNTER] && [file size $DPBOX_COUNTER]==8} {
    # open counter file
    set fd [open $DPBOX_COUNTER RDWR]
    scan [read $fd 8] "%x" counter
    seek $fd 0 start
} else {
    # Counter don't exist or is not correct: create counter !!
    if {[catch "open $DPBOX_COUNTER {RDWR CREAT} 0644" fd]} {
	printerror "[putslang 40]: $fd"
	return
    }
    set counter 1
}
puts -nonewline $fd [format "%08x" [expr $counter+1]]

close $fd

# Did client sent the cookie? (it is stored in par(DP) var)
if {![info exist par(DP)]} {
    # browser did not sent me the cookie! => autentication

    if {![info exist par(c)]} {
	printheader ""
	askpassword
    }
    if {[string equal "control" "$par(c)"]} {
	# did client provided both username and password
	if {![info exist par(u)] || ![info exist par(p)]} {
	    printheader ""
	    puts $fdout "<HR><BIG>[putslang 39]</BIG><BR>[putslang 35]<HR><BR>"
	    askpassword
	}
	# parameters c and p exists: they may NOT be empty!
	if {[string length "$par(u)"]>0 && [string length "$par(p)"]>0} {
	    verifyaccess
	} else {
	    # client does not provide both callsign and password
	    printheader ""
	    puts $fdout "<BIG>[putslang 39]</BIG>"
	    askpassword
	}
    } elseif {[string equal "regform" "$par(c)"]} { ;# user request to compile registration form
	printheader ""
	regform
    } elseif {[string equal "register" "$par(c)"]} { ;# send user information to DPBOX_ADMIN
	if {[string bytelength $par(u)]<4 || [string bytelength $par(p)]<5 || [string bytelength $par(firstname)]<3 || [string bytelength $par(lastname)]<4 || [string bytelength $par(qth)]<4} { ;# form not compiled correctly
	    printheader ""
	    puts $fdout "<div class=\"auth\"><big>Please compile the form in all its part!</big></div><br><br>"
	    regform
	} else {
	    sendmail "New user registration request" "Request to access dpbox at [clock format [clock seconds] -format "%d/%m/%Y %T"]\ncallsign:$par(u)\npassword:$par(p)\nName:    $par(firstname)\nLastname:$par(lastname)\nQTH:     $par(qth)\nemail:   $par(email)\n\nPlease control user information and grant permission using command\n\n    dpaccess_pass $par(u) $par(p) $par(firstname)\n\nRemember to hold messages sent from $par(u) into ~dpbox/system/reject.box\n"
	    printheader ""
	    registerok
	}
    } else {
	;# client does not provide neither cookie or command
	printheader ""
	askpassword
    }
} else {
    # browser sent me the cookie! Note: if client provide me also callsign and password, use it for identifications!
    if {[info exist par(u)] && [info exist par(p)] && [string length par(u)]>=4 && [string length par(p)]>0} {
	verifyaccess
    }
    set par(u) "[lindex [split $par(DP) ":"] 0]"
    set par(p) "[lindex [split $par(DP) ":"] 1]"
    set par(lasttime) [lindex [split $par(DP) ":"] 2]
    if {![string is integer -strict $par(lasttime)]} {
	set par(lasttime) 0
    }
    if {[string length "$par(u)"]<4 || [string length "$par(p)"]<4} {
	printheader ""
	askpassword
    }
    access decode "$par(u)" "$par(p)"
    # user is identify correctly. Which command have he sent?
    if {![info exist par(c)]} {set par(c) "menu" }
    set message ""
    if {[string equal "menu" "$par(c)"]} {
	set message "DP=$par(u):$par(p):[clock seconds]"
    } elseif {[string equal "selstyle" "$par(c)"] && [info exist par(newstyle)]} {
	set style "$par(newstyle)"
	set message "STYLE=$style"
    }
    printheader "$message"
}
    if {[string equal "listp" "$par(c)"]} { ;# list private messages
	puts $fdout "</CENTER><HR><PRE>"
	puts $fdout "[exec $DP_GATE -u $par(u) -s $DP_SOCKET -i -c "L 1-" | $DP_CODECONV | $DP_PRIVLINKS -list "$par(u)" "$par(p)"]</PRE><HR>"
	printfooter -1
    } elseif {[string equal "listb" "$par(c)"]} { ;# list messages in the specified board
	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u $par(u) -s $DP_SOCKET -i -c "L $par(b) 1-" | $DP_CODECONV | $DP_PRIVLINKS -listb "$par(u)" "$par(p)" "BR&b=$par(b)"]</PRE><HR>"
	printfooter -1
    } elseif {[string equal "check" "$par(c)"]} {	;# list all bulletins since last login
	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c CS | $DP_CODECONV | $DP_PRIVLINKS -check $par(u) "$par(p)"]</PRE><HR>"
	exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c Q >/dev/null
	printfooter -1
    } elseif {[string equal "check100" "$par(c)"]} { ;# send the list of last 100 bulletins
    	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "CS 100" | $DP_CODECONV | $DP_PRIVLINKS -check "$par(u)" "$par(p)"]</PRE><HR>"
	printfooter -1
    } elseif {[string equal "check500" "$par(c)"]} { ;# list last 500 bulletins
	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "CS 500" | $DP_CODECONV | $DP_PRIVLINKS -check "$par(u)" "$par(p)"]</PRE><HR>"
	printfooter -1
    } elseif {[string equal "dir" "$par(c)"]} { ;# list all available boards
	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "D I" | $DP_CODECONV | $DP_PRIVLINKS -dir "$par(u)" "$par(p)"]</PRE><HR>"
	printfooter -1
    } elseif {[string equal "BR" "$par(c)"]} { ;# read bulletin
	puts $fdout "<TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=\"$DPACCESS_PATH?c=rep&b=$par(b)&n=$par(n)\">[putslang 41]</A>\
<TD><A HREF=\"$DPACCESS_PATH?c=com&b=$par(b)&n=$par(n)\">[putslang 42]</A>\
</TABLE><HR></CENTER><PRE>"
	puts $fdout "[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "R $par(b) $par(n)" | $DP_CODECONV]\
</PRE><HR><center><TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=\"$DPACCESS_PATH?c=rep&b=$par(b)&n=$par(n)\">[putslang 41]</A>\
<TD><A HREF=\"$DPACCESS_PATH?c=com&b=$par(b)&n=$par(n)\">[putslang 42]</A></TABLE></center>"
	printfooter -2
    } elseif {[string equal "PR" "$par(c)"]} { #; Read private message
	puts $fdout "<TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=\"$DPACCESS_PATH?c=rep&b=$par(u)&n=$par(n)\">[putslang 41]</A>\
</TABLE><HR></CENTER><PRE>"
	puts $fdout "[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "R $par(u) $par(n)" | $DP_CODECONV ]\
</PRE><HR><CENTER><TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=\"$DPACCESS_PATH?c=rep&b=$par(u)&n=$par(n)\">[putslang 41]</A>\
</TABLE></CENTER>"
	printfooter -2
    } elseif {[string equal "rep" "$par(c)"]} { ;# reply with a private message
	puts $fdout "<FORM ACTION=$DPACCESS_PATH METHOD=post><INPUT TYPE=hidden NAME=b VALUE=$par(b)>\
<INPUT TYPE=hidden NAME=n VALUE=$par(n)><INPUT TYPE=hidden NAME=c VALUE=sendrep>\
Message body:<BR><TEXTAREA name=body rows=10 cols=78 wrap=physical></TEXTAREA><BR>\
<INPUT TYPE=submit VALUE=\"[putslang 43]\"></FORM></BODY></HTML>"
    } elseif {[string equal "sendrep" "$par(c)"]} { ;# send the private message
	set filename "/tmp/dpaccess[pid]"
	exec $DP_GATE -u "$par(u)" -c "L $par(b) $par(n) :" | $DP_PRIVLINKS -rep "$par(u)" "$par(p)" >$filename
	set fd [open $filename {WRONLY APPEND}]
	parsebody "$par(body)"
	puts $fd "$message\n\n\n[putslang 900]"
	close $fd
	file rename $filename "/var/spool/dpbox/newmail/import.dpaccess[pid]"
	puts $fdout "<B>[putslang 45]</B><BR><table align=center class=\"reply\"><TR>\
<TD><A HREF=JavaScript:window.history.go(-3)>[putslang 46]</A></TABLE>"
	printfooter -4
    } elseif {[string equal "com" "$par(c)"]} { ;# reply with a bulletin
	puts $fdout "<FORM ACTION=$DPACCESS_PATH METHOD=post><INPUT TYPE=hidden NAME=b VALUE=$par(b)><INPUT TYPE=hidden NAME=n VALUE=$par(n)>\
<INPUT TYPE=hidden NAME=c VALUE=sendcom>[putslang 47]<br>\
<TEXTAREA name=body rows=10 cols=78 wrap=physical></TEXTAREA><BR><INPUT TYPE=submit VALUE=\"[putslang 43]\"></FORM></BODY></HTML>"
    } elseif {[string equal "sendcom" "$par(c)"]} { ;# send the private message
	set filename "/tmp/dpaccess[pid]"
	exec $DP_GATE -u "$par(u)" -c "L $par(b) $par(n) :" | $DP_PRIVLINKS -com "$par(u)" "$par(p)" >$filename
	set fd [open $filename {WRONLY APPEND}]
	parsebody "$par(body)"
	puts $fd "$message\n\n\n[putslang 900]"
	close $fd
	file rename $filename "/var/spool/dpbox/newmail/import.dpaccess[pid]"
	puts $fdout "<B>[putslang 45]</B><BR><TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=JavaScript:window.history.go(-3)>[putslang 46]</A></TABLE>"
	printfooter -4
    } elseif {[string equal "create" "$par(c)"]} { ;#create a new message
	puts $fdout "<FORM ACTION=$DPACCESS_PATH METHOD=post><INPUT TYPE=hidden NAME=c VALUE=send>\
<TABLE class=\"create\">\
<TR><TH align=right>[putslang 48] &nbsp; </TH>\
<TD><INPUT TYPE=text NAME=address SIZE=16 MAXLENGTH=40></TD></TR>\
<TR><TH align=right>[putslang 49] </TH><TD><INPUT TYPE=text NAME=subject SIZE=40 MAXLENGTH=40></TD></TR>\
<TR><TH colspan=2>[putslang 47] </TH></TR>\
<TR><TD COLSPAN=2><TEXTAREA NAME=body ROWS=10 COLS=78 wrap=physical></TEXTAREA></TD></TR>\
<TR><TD COLSPAN=2 ALIGN=CENTER><INPUT TYPE=submit VALUE=\"[putslang 43]\"></TD></TR></TABLE><BR></FORM></BODY></HTML>"
    } elseif {[string equal "send" "$par(c)"]} { ;#send the message created with create
	set filename "/tmp/dpaccess[pid]"
	set fd [open $filename {WRONLY CREAT}] ;#write the import file
	puts $fd "$DPBOX_CALLSIGN\n$par(u)"
	set message [split $par(address) "@"]
	set par(board) [lindex $message 0]
	set par(zone) [lindex $message 1]
	parsebody "$par(body)"
	puts $fd "$par(board)\n$par(zone)\n\n\n$par(subject)                                      \n$message\
\n\n[putslang 900]\n"
        close $fd
        file rename $filename "/var/spool/dpbox/newmail/import.dpaccess[pid]"
        puts $fdout "<B>Message sent!</B><BR><TABLE class=\"reply\"><TR ALIGN=center>\
<TD><A HREF=JavaScript:window.history.go(-1)>[putslang 50]</A></TABLE>"
	printfooter -2
    } elseif {[string equal "u" "$par(c)"]} { ;# show connected users
	puts $fdout "<H2>Connected users:</H2></CENTER><PRE>[exec $DP_GATE -u "$par(u)" -c "U PATH" | $DP_CODECONV ]</PRE><CENTER>"
	printfooter -1
    } elseif {[string equal "v" "$par(c)"]} { ;# show DPBOX version
	puts $fdout "<H2>[putslang 27]:</H2></CENTER><PRE>[exec $DP_GATE -u "$par(u)" -c "V +" | $DP_CODECONV ]</PRE><CENTER>"
	printfooter -1
    } elseif {[string equal "bbsinfo" "$par(c)"]} { ;# show BBS informations
	puts $fdout "<H2>[putslang 22]:</H2></CENTER><PRE>[exec $DP_GATE -u "$par(u)" -c "BBS $par(bbs)" | $DP_CODECONV ]</PRE><CENTER>"
	printfooter -1
    } elseif {[string equal "userinfo" "$par(c)"]} { ;# show USER informations
	puts $fdout "</CENTER><HR SIZE=5 NOSHADE><H2>[putslang 26] <b>$par(user)</b>:</H2>\
<PRE>[exec $DP_GATE -u "$par(u)" -c "u $par(user)" | $DP_CODECONV ]\n</PRE><HR SIZE=5 NOSHADE>\
<H2>[putslang 51]</H2><PRE>[exec $DP_GATE -u "$par(u)" -c "u u" | $DP_CODECONV ]</PRE>\
<HR SIZE=5 NOSHADE><CENTER>"
	printfooter -1
    } elseif {[string equal "seekh" "$par(c)"]} { ;# seek messages using C <key
    	puts $fdout "</CENTER><HR><PRE>[exec $DP_GATE -u "$par(u)" -s $DP_SOCKET -i -c "C <$par(key)" | $DP_CODECONV | $DP_PRIVLINKS -check "$par(u)" "$par(p)"]</PRE><HR>"
	printfooter -1
    } else {
	# default: print the menu
	printmenu
    }
    close $fdout
    printstat
}








